import numpy as np
from ._CFunctions import _CWithinMP

###### File created automatically using PopulateCtypes ######

def WithinMP(x, y, z, Rsm):

	#Convert input variables to appropriate numpy dtype:
	_x = np.float64(x)
	_y = np.float64(y)
	_z = np.float64(z)
	_Rsm = np.float64(Rsm)
	res = _CWithinMP(_x, _y, _z, _Rsm)

	return res



import numpy as np

def PlotShueMP(fig,Rsm,Alpha,color=[0.0,0.0,1.0],lw=2,zorder=10,NoonTop=True):
	

	theta = np.pi*(np.arange(359)-179)/180.0
	r = Rsm*(2.0/(1+np.cos(theta)))**Alpha
	x = r*np.cos(theta)
	z = r*np.sin(theta)
	
	if NoonTop:
		fig.plot(z,x,color=color,lw=lw,zorder=zorder,label='Magnetopause')
	else:
		fig.plot(x,z,color=color,lw=lw,zorder=zorder,label='Magnetopause')



def PlotShueMPYZ(fig,Rsm,Alpha,color=[0.0,0.0,1.0],lw=2,zorder=10):

	theta = np.pi*(np.arange(359)-179)/180.0
	r = Rsm*(2.0)**Alpha
	y = r*np.cos(theta)
	z = r*np.sin(theta)
	
	fig.plot(y,z,color=col,lw=lw,zorder=zorder,label='Magnetopause')


def PlotMercuryMP(fig,Rsm=1.42,Alpha=0.5,color=[0.0,0.0,1.0],lw=2,zorder=10,NoonTop=True):
	PlotShueMP(fig,Rsm,Alpha,color,lw,zorder,NoonTop)

def PlotMercuryMPYZ(fig,Rsm=1.42,Alpha=0.5,color=[0.0,0.0,1.0],lw=2,zorder=10):
	PlotShueMPYZ(fig,Rsm,Alpha,color,lw,zorder)

def PlotKT17MercuryMP(fig,Rsun=0.39,DistInd=50.0,color=[0.0,0.0,1.0],lw=2,zorder=10,NoonTop=True):
	f=2.06873-0.00279*DistInd
	Rsm=f*Rsun**(1.0/3.0) 	
	
	PlotShueMP(fig,Rsm,0.5,color,lw,zorder,NoonTop)

def PlotKT17MercuryMPYZ(fig,Rsun=0.39,DistInd=50.0,color=[0.0,0.0,1.0],lw=2,zorder=10):
	f=2.06873-0.00279*DistInd
	Rsm=f*Rsun**(1.0/3.0) 	
	PlotShueMPYZ(fig,Rsm,0.5,color,lw,zorder)



import numpy as np
from ._CFunctions import _Ckt17Barray

###### File created automatically using PopulateCtypes ######

def ModelField(x, y, z, Params=[1.42,7.37,2.16]):

	#Convert input variables to appropriate numpy dtype:
	_n = np.int32(np.size(x))
	_x = np.array([x]).astype("float64").flatten()
	_y = np.array([y]).astype("float64").flatten()
	_z = np.array([z]).astype("float64").flatten()
	_Bx = np.zeros(_n,dtype="float64")
	_By = np.zeros(_n,dtype="float64")
	_Bz = np.zeros(_n,dtype="float64")
	
	Par = np.array(Params)	
	shP = Par.shape
	if np.size(shP) == 2:
		if shP[1] == _n:
			Par = Par.T
			shP = Par.shape
		_nP = np.int32(shP[1]) # number of parameters for each trace
	else:
		_nP = np.int32(shP[0])
	_Plen = np.int32(np.size(Par)) # total number of parameters
	
	_Params = Par.flatten().astype('float64')
	
	_Ckt17Barray(_n, _x, _y, _z, _Bx, _By, _Bz, _nP, _Plen, _Params)

	return (_Bx,_By,_Bz)



import numpy as np
import matplotlib.pyplot as plt
from .TraceField import TraceField
from .PlotMP import PlotKT17MercuryMP,PlotMercuryMP
from .PlotPlanet import PlotPlanetXZ

def TestTrace(degstep=5.0,Params=(0.39,50.0)):
	n = np.int32(360.0/degstep)
	a = np.arange(n)*np.pi*2.0/n
	
	x = 1.01*np.cos(a)
	y = np.zeros(n)
	z = 1.01*np.sin(a)
	
	T = TraceField(x,y,z,Params=Params)
	
	fig = plt
	fig.figure()
	PlotPlanetXZ(fig,Center=[0.0,0.0,-0.196],NoonTop=False)
	
	if np.size(Params) == 2:
		PlotKT17MercuryMP(fig,Params[0],Params[1],NoonTop=False)
	else:	
		PlotMercuryMP(fig,Params[0],0.5,NoonTop=False)
	fig.plot(T.x.T,T.z.T,color=[0.0,0.0,0.0],lw=1.0)
	fig.axis([2.0,-10.0,-4.0,4.0])
	ax = fig.gca()
	ax.set_aspect(1.0)



import numpy as np

def PlotPlanetXZ(fig,R=1.0,Center=[0.0,0.0,0.0],zorder=10,NoBlack=False,NoonTop=True):
	

	a = 2*np.pi*np.arange(361,dtype='float32')/360
	x = R*np.sin(a) + Center[0]
	z = R*np.cos(a) + Center[2]
	
	if NoonTop:	
		fig.fill(z,x,color=[1.0,1.0,1.0],zorder=zorder)
		fig.plot(z,x,color=[0,0,0],zorder=zorder+1)
		if NoBlack == False:
			fig.fill(z[180:360],x[180:360],color=[0.0,0.0,0.0],zorder=zorder+1)
	else:
		fig.fill(x,z,color=[1.0,1.0,1.0],zorder=zorder)
		fig.plot(x,z,color=[0,0,0],zorder=zorder+1)
		if NoBlack == False:
			fig.fill(x[180:360],z[180:360],color=[0.0,0.0,0.0],zorder=zorder+1)


def PlotPlanetXY(fig,R=1.0,Center=[0.0,0.0,0.0],zorder=10,NoBlack=False,NoonTop=True):
	a = 2*np.pi*np.arange(361,dtype='float32')/360
	x = R*np.sin(a) + Center[0]
	y = R*np.cos(a) + Center[1]
	
	if NoonTop:	
		fig.fill(y,x,color=[1.0,1.0,1.0],zorder=zorder)
		fig.plot(y,x,color=[0,0,0],zorder=zorder+1)
		if NoBlack == False:
			fig.fill(y[180:360],x[180:360],color=[0.0,0.0,0.0],zorder=zorder+1)
	else:
		fig.fill(x,y,color=[1.0,1.0,1.0],zorder=zorder)
		fig.plot(x,y,color=[0,0,0],zorder=zorder+1)
		if NoBlack == False:
			fig.fill(x[180:360],y[180:360],color=[0.0,0.0,0.0],zorder=zorder+1)

def PlotPlanetYZ(fig,R=1.0,Center=[0.0,0.0,0.0],Side='day',zorder=10,NoFill=False,Color=[0.0,0.0,0.0],linestyle='-'):
	
	a = 2*np.pi*np.arange(361,dtype='float32')/360
	y = R*np.sin(a) + Center[1]
	z = R*np.cos(a) + Center[2]
	
	if NoFill == False:	
		if Side == 'day':
			fig.fill(y,z,color=[1.0,1.0,1.0],zorder=zorder)
		else:
			fig.fill(y,z,color=[0.0,0.0,0.0],zorder=zorder)
	fig.plot(y,z,color=Color,zorder=zorder+1,linestyle='-')




import setuptools
from setuptools.command.install import install
import os

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="KT17",
    version="0.0.1",
    author="Matthew Knight James",
    author_email="mattkjames7@gmail.com",
    description="KT17/KT14 magnetic field model for Mercury written in C++ with a Python wrapper. See Korth et al., 2015 (JGR) and Korth et al., 2017 (GRL) for more details.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mattkjames7/KT17",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
    ],
    install_requires=[
		'numpy',
		'scipy',
		'matplotlib',
	],
	include_package_data=True,
)






import numpy as np
from ._CFunctions import _Ckt17MultiTrace
from scipy.interpolate import InterpolatedUnivariateSpline

###### File created automatically using PopulateCtypes ######
'''
LimTypes include:
	Confining to a box: -6 < x < 2, -4 < y < 4, -4 < z < 4
	Confine to box and planet's surface
	Confine to just outside the planet
	Confine to within MP, outside planet and within 10Rm
	Default is trace to surface and to MP

New LimType options
	Each = 1 bit
	Default: 1111000 (Big Endian) = 15
	MP						+1
	Planet					+2
	Dipole of Planet		+4
	Tail Limit at 10Rm		+8
	Core					+16
	Core Dipole				+32
	Box						+64
	


'''


class TraceField(object):
	def __init__(self,x0,y0,z0,Params=[1.42,7.37,2.16],maxlen=1000,
				initstep=0.01,maxstep=0.05,LimType=15,
				FlattenSingleTraces=True,SmoothSurfaceTransition=False,
				AssumeOutsidePlanet=True):

		'''
		The TraceField object performs field line traces from initial
		positions provided using the KT17/KT14 model.
		
		Inputs:
			x0,y0,z0: Scalars or arrays containing the x-MSM (Mercury Solar
					Magnetic) coordinate of the	starting point for the trace.
			Params: 2 or 3 element array, list or tuple of model parameters.
					For the KT14 model - Params=[Rss,T1,T2] where Rss is 
					the	subsolar magnetopause distance in Rm, T1 is the 
					magnitude of the tail disk current and T2 is the 
					magnitude of the quasi-harris sheet. By default,
					Params = [1.42,7.37,2.16].
					For the KT17 model - Params=[Rsun,DistIndex], where
					Rsun is the distance of Mercury from the Sun in AU,
					DistIndex is the disturbance index (0-100) as calculated
					in Anderson et al., 2013.
			maxlen: Maximum number of steps used to trace the field.
			initstep: Initial step size for trace in Rm.
			maxstep: Maximum length of a trace step in Rm.
			LimType: This tells the C++ code where to terminate the trace.
					The different options can be enabled by setting the 
					appropriate bit of and 8-bit integer to 1, where the
					options are:
						MP						+1 	(Stop if magnetopause is reached)
						Planet					+2	(Stop at planetary surface)
						Dipole of Planet		+4	(Stop 1 Rm from centre of planetary dipole)
						Tail Limit at 10Rm		+8	(Stop if trace reaches x MSM < -10 (in the magnetotail))
						Core					+16	(Stop at the iron core of Mercury (R~2030km))
						Core Dipole				+32 (Stop at a distance from the centre of the dipole equivalent to the size of Mercury's core)
						Box						+64 (Stop within a box where -6 < x < 2, -4 < y < 4, -4 < z < 4)
					Default: 1111000 (Big Endian) = 15 (stop at MP, Planetary surface in the south, 1Rm from the dipole in the north and 10Rm down-tail)
			FlattenSingleTraces: Flattens the position and field magnitude 
					arrays created if only a single trace is performed.
			SmoothSurfaceTransition: Smooths the transition from within
					the planets mantle/crust to outside of the planet
					by inserting extra points along the trace within a small
					distance of the surface. The InPlanet attribute will 
					vary smoothly from 1.0 to 0.0 where the field trace 
					is near to the crust. Requires AssumeOutsidePlanet=False
					and traces to terminate at the planets core.
			AssumeOutsidePlanet: Assumes the the magnetosphere extends 
					all the way to the surface of the core - sets 
					InPlanet[:] = 0.0.
					
			
		Attributes:
			nstep: (n,) array with number of steps taken in each trace.
			x,y,z:	(n,maxlen) arrays containing positions in MSM of the
				field line along the trace, where n is the number of traces.
				For trace i, x[i,0:nstep[i]],y[i,0:nstep[i]],z[i,0:nstep[i]] 
				contain real values positions and x[i,nstep[i]:],
				y[i,nstep[i]:],z[i,nstep[i]:] = NAN. If n==1 and 
				FlattenSingleTraces=True, then the dimensions are reduced 
				to (maxlen,).
			Bx,By,Bz: Magnetic field along the trace in nT.
			Rmsm: Radial distance of trace points in Rm in the MSM
				coordinate system (centred upon the magnetic dipole).
			Rmso: Radial distance of each trace point in Rm in the MSO
				(Mercury Solar Orbital) coordinate system, where
				xMSO,yMSO = xMSM,yMSM and zMSO (centered on planet) = 
				zMSM+0.19.
			InPlanet: Floating point array for each trace, where InPlanet
				= 1.0 inside the planet and InPlanet = 0.0 outside. This
				is useful when modeling MHD waves at Mercury.
				
			MlatN,MlatS: Magnetic latitudes of the northern and southern fieldline footprints.
			MltN,MltS: Magnetic local times of the northern and southern fieldline footprints.
			GlatN,GlatS: Hermeographic (is that a word?) latitudes of the northern and southern fieldline footprints.
			GltN,GltS: Hermeographic local times of the northern and southern fieldline footprints.
			
			MlatNcore,MlatScore: Magnetic latitudes of the northern and southern core fieldline footprints.
			MltNcore,MltScore: Magnetic local times of the northern and southern core fieldline footprints.
			GlatNcore,GlatScore: Hermeographic (is that a word?) latitudes of the northern and southern core fieldline footprints.
			GltNcore,GltScore: Hermeographic local times of the northern and southern core fieldline footprints.
			
			MltE: Magnetic local time at the magnetic equatorial (zMSM=0) footprint.
			Lshell: Radial distance of magnetic equatorial footprint.
			FlLen: Total length of field line (NAN if not closed)
			FlLencore: Total length of field line traced to the core.
					
		'''
		

		#Convert input variables to appropriate numpy dtype:
		_x0 = np.array([x0]).astype("float64").flatten()
		_y0 = np.array([y0]).astype("float64").flatten()
		_z0 = np.array([z0]).astype("float64").flatten()
		_n = np.int32(np.size(x0))
		_maxlen = np.int32(maxlen)
		_initstep = np.float64(initstep)
		_maxstep = np.float64(maxstep)
		_nstep = np.zeros(_n,dtype='int32')
		_x = np.zeros(_n*_maxlen,dtype='float64')+np.nan
		_y = np.zeros(_n*_maxlen,dtype='float64')+np.nan
		_z = np.zeros(_n*_maxlen,dtype='float64')+np.nan
		_bx = np.zeros(_n*_maxlen,dtype='float64')+np.nan
		_by = np.zeros(_n*_maxlen,dtype='float64')+np.nan
		_bz = np.zeros(_n*_maxlen,dtype='float64')+np.nan
		_Rmsm = np.zeros(_n*_maxlen,dtype='float64')+np.nan
		_Rmso = np.zeros(_n*_maxlen,dtype='float64')+np.nan
		_FP = np.zeros(_n*20,dtype='float64')+np.nan
		_LimType = np.int32(LimType)
		_Params = np.float64(Params)
		_nParams = np.int32(_Params.size)
		#call the C++ function 
		_Ckt17MultiTrace(_x0, _y0, _z0, _n, _maxlen, _initstep, _maxstep,
						_LimType, _nParams, _Params, _nstep, _x, _y, _z,
						_bx, _by, _bz, _Rmsm, _Rmso, _FP)

		#reshape the footprints
		_FP = _FP.reshape((_n,20))
		
		fpnames = ['MlatN','MlatS','GlatN','GlatS','MltN','MltS','GltN',
					'GltS','MlatNcore','MlatScore','GlatNcore','GlatScore',
					'MltNcore','MltScore','GltNcore','GltScore','Lshell',
					'MltE','FlLen','FlLencore']

		self.n = _n
		if _n == 1 and FlattenSingleTraces:
			self.nstep = _nstep[0]
			self.x = _x
			self.y = _y
			self.z = _z
			self.Bx = _bx
			self.By = _by
			self.Bz = _bz
			self.Rmsm = _Rmsm
			self.Rmso = _Rmso
			for i in range(0,20):
				setattr(self,fpnames[i],_FP[0,i])

		else:		
			self.nstep = _nstep
			self.x = _x.reshape((_n,_maxlen))
			self.y = _y.reshape((_n,_maxlen))
			self.z = _z.reshape((_n,_maxlen))
			self.Bx = _bx.reshape((_n,_maxlen))
			self.By = _by.reshape((_n,_maxlen))
			self.Bz = _bz.reshape((_n,_maxlen))
			self.Rmsm = _Rmsm.reshape((_n,_maxlen))
			self.Rmso = _Rmso.reshape((_n,_maxlen))
			for i in range(0,20):
				setattr(self,fpnames[i],_FP[:,i])


		#make a copy of the original traces before tey are butchered by the options 
		self.nstep_full = np.copy(self.nstep)
		self.x_full = np.copy(self.x)
		self.y_full = np.copy(self.y)
		self.z_full = np.copy(self.z)
		self.Bx_full = np.copy(self.Bx)
		self.By_full = np.copy(self.By)
		self.Bz_full = np.copy(self.Bz)	
		self.Rmsm_full = np.copy(self.Rmsm)	
		self.Rmso_full = np.copy(self.Rmso)	
		
		
		bits = np.array(list('{:032b}'.format(LimType))[::-1]).astype('bool8')
		if bits[4]:
			Rcutoff = 0.832
			Ruse = self.Rmso_full
		elif bits[5] and not bits[4]:
			Rcutoff = 0.832
			Ruse = self.Rmsm_full
		elif bits[2] and not bits[1]:
			Rcutoff = 1.0
			Ruse = self.Rmsm_full
		else:
			Rcutoff = 1.0
			Ruse = self.Rmso_full
		
		if _n == 1 and FlattenSingleTraces:
			good = np.where(np.isfinite(self.x) & (Ruse >= Rcutoff))[0]
			if good.size > 0:
				if good[0] > 0:
					if np.isfinite(Ruse[good[0]-1]):
						good = np.append(good[0]-1,good)
				if good[-1] < Ruse.size-1:
					if np.isfinite(Ruse[good[-1]+1]):
						good = np.append(good,good[-1]+1)
				
				

				self.x = self.x_full[good]
				self.y = self.y_full[good]
				self.z = self.z_full[good]
				self.Bx = self.Bx_full[good]
				self.By = self.By_full[good]
				self.Bz = self.Bz_full[good]
				self.Rmsm = self.Rmsm_full[good]
				self.Rmso = self.Rmso_full[good]
				
				
				self.nstep = good.size		
				
				good = np.where(np.isfinite(self.x) & (self.Rmso >= 1.0))[0]
				if good[0] > 0:
					if np.isfinite(self.Rmso[good[0]-1]):
						good = np.append(good[0]-1,good)
				if good[-1] < self.Rmso.size-1:
					if np.isfinite(self.Rmso[good[-1]+1]):
						good = np.append(good,good[-1]+1)		
				self.InPlanet = np.ones(self.nstep,dtype='float32')
				self.InPlanet[good] = 0.0
			else:
				self.InPlanet = np.zeros(self.nstep,dtype='float32')
		else:
			for i in range(0,_n):
				good = np.where(np.isfinite(self.x[i]) & (Ruse[i] >= Rcutoff))[0]
				bad = np.where((np.isfinite(self.x[i]) == False) | (Ruse[i] < Rcutoff))[0]	
				gd = np.arange(good.size)
				bd = np.arange(bad.size) + good.size
				self.nstep[i] = good.size	
				self.x[i,gd] = self.x_full[i,good]
				self.x[i,bd] = np.nan
				self.y[i,gd] = self.y_full[i,good]
				self.y[i,bd] = np.nan
				self.z[i,gd] = self.z_full[i,good]
				self.z[i,bd] = np.nan				
				self.Bx[i,gd] = self.Bx_full[i,good]
				self.Bx[i,bd] = np.nan
				self.By[i,gd] = self.By_full[i,good]
				self.By[i,bd] = np.nan
				self.Bz[i,gd] = self.Bz_full[i,good]
				self.Bz[i,bd] = np.nan						
				self.Rmsm[i,gd] = self.Rmsm_full[i,good]
				self.Rmsm[i,bd] = np.nan						
				self.Rmso[i,gd] = self.Rmso_full[i,good]
				self.Rmso[i,bd] = np.nan						

			self.InPlanet = (self.Rmso < 1.0).astype('float32')
		
		
		if AssumeOutsidePlanet:
			self.InPlanet[:] = 0.0			
		

		if SmoothSurfaceTransition and _n == 1 and FlattenSingleTraces:
			# Here we need to add some extra points along the field line in order to  
			#smooth the transition between inside the planet and outside
			J = np.where(np.abs(self.InPlanet[1:]-self.InPlanet[:-1]) == 1)[0]
			J = J[np.where((J != 0) & (J != self.nstep-2))[0]]
			nJ = J.size
			if nJ > 0:
				#get distance along field line
				s0 = np.zeros(self.nstep,dtype='float32')
				for i in range(1,self.nstep):
					s0[i] = s0[i-1] + np.sqrt((self.x[i]-self.x[i-1])**2 + (self.y[i]-self.y[i-1])**2 + (self.z[i]-self.z[i-1])**2)
				
				nInsert = 19 # number of extra points to add between steps
				nstep_new = self.nstep + nJ*nInsert
				
				#get new distance array along field line
				s1 = np.zeros(nstep_new,dtype='float32')
				new_InPlanet = np.zeros(nstep_new,dtype='float32')
				p = 0
				for i in range(0,nJ+1):
					if i == 0:
						j0 = 0
						j1 = J[0]
					elif i == nJ:
						j0 = J[-1]+1
						j1 = self.nstep-1
					else:
						j0 = J[i-1]+1
						j1 = J[i]
					
					for j in range(j0,j1+1):
						s1[p] = s0[j]
						new_InPlanet[p] = self.InPlanet[j]
						p += 1
						
					if i < nJ:
						ds = s0[J[i]+1] - s0[J[i]]
						dip = self.InPlanet[J[i]+1] - self.InPlanet[J[i]]
						for j in range(0,nInsert):
							dj = (j+1)/(nInsert+1)
							s1[p] = s0[J[i]] + ds*dj
							new_InPlanet[p] = self.InPlanet[J[i]] + dip*(3*dj**2-2*dj**3)
							p+=1
				self.InPlanet = new_InPlanet
				self.s0 = s0
				self.s1 = s1
				Params = ['x','y','z','Bx','By','Bz','Rmsm','Rmso']
				for par in Params:
					tmp0 = np.copy(getattr(self,par))[0:self.nstep]
					
					f = InterpolatedUnivariateSpline(s0,tmp0)
					tmp1 = f(s1).astype('float32')
					
					setattr(self,par,tmp1)
				self.nstep = nstep_new



import numpy as np
import ctypes as ct
import os
import platform

Arch = platform.architecture()[0]
if Arch == '64bit':
	libkt17 = ct.CDLL(os.path.dirname(__file__)+"/__data/libkt17/libkt17_64.so")
elif Arch == '32bit':
	libkt17 = ct.CDLL(os.path.dirname(__file__)+"/__data/libkt17/libkt17_32.so")
else:
	print('Architecture ({:s}) not found'.format(Arch))



_CWithinMP = libkt17.WithinMP
_CWithinMP.argtypes = [ct.c_double, ct.c_double, ct.c_double, ct.c_double]
_CWithinMP.restype = ct.c_bool

_Ckt17DipoleField = libkt17.kt17DipoleField
_Ckt17DipoleField.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17DipoleField.restype = None

_Ckt17DipoleShield = libkt17.kt17DipoleShield
_Ckt17DipoleShield.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17DipoleShield.restype = None

_Ckt17DipoleB = libkt17.kt17DipoleB
_Ckt17DipoleB.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17DipoleB.restype = None

_Ckt17DiskThickness = libkt17.kt17DiskThickness
_Ckt17DiskThickness.argtypes = [ct.c_double, ct.c_double]
_Ckt17DiskThickness.restype = ct.c_double

_Ckt17DiskField = libkt17.kt17DiskField
_Ckt17DiskField.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17DiskField.restype = None

_Ckt17DiskShield = libkt17.kt17DiskShield
_Ckt17DiskShield.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17DiskShield.restype = None

_Ckt17DiskB = libkt17.kt17DiskB
_Ckt17DiskB.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_double]
_Ckt17DiskB.restype = None

_Ckt17QuasiHarrisThickness = libkt17.kt17QuasiHarrisThickness
_Ckt17QuasiHarrisThickness.argtypes = [ct.c_double, ct.c_double]
_Ckt17QuasiHarrisThickness.restype = ct.c_double

_Ckt17QuasiHarrisField = libkt17.kt17QuasiHarrisField
_Ckt17QuasiHarrisField.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17QuasiHarrisField.restype = None

_Ckt17QuasiHarrisShield = libkt17.kt17QuasiHarrisShield
_Ckt17QuasiHarrisShield.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17QuasiHarrisShield.restype = None

_Ckt17QuasiHarrisB = libkt17.kt17QuasiHarrisB
_Ckt17QuasiHarrisB.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_double]
_Ckt17QuasiHarrisB.restype = None

_Ckt17B = libkt17.kt17B
_Ckt17B.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_double, ct.c_double, ct.c_double]
_Ckt17B.restype = None

_Ckt17Barray = libkt17.kt17Barray
_Ckt17Barray.argtypes = [ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17Barray.restype = None

_Ckt17 = libkt17.kt17
_Ckt17.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_double, ct.c_double]
_Ckt17.restype = None

# _Ckt17array = libkt17.kt17array
# _Ckt17array.argtypes = [ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
# _Ckt17array.restype = None

_CUnitVector = libkt17.UnitVector
_CUnitVector.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_CUnitVector.restype = None

_CReverseElements = libkt17.ReverseElements
_CReverseElements.argtypes = [np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int]
_CReverseElements.restype = None

_CInsideTraceLims = libkt17.InsideTraceLims
_CInsideTraceLims.argtypes = [ct.c_double, ct.c_double, ct.c_double,np.ctypeslib.ndpointer(ct.c_bool,flags="C_CONTIGUOUS"), ct.c_int, ct.c_double]
_CInsideTraceLims.restype = ct.c_bool

_CNorthSouthFLs = libkt17.NorthSouthFLs
_CNorthSouthFLs.argtypes = [np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_int,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_int,flags="C_CONTIGUOUS")]
_CNorthSouthFLs.restype = None

_Clinterp = libkt17.linterp
_Clinterp.argtypes = [ct.c_double, ct.c_double, ct.c_double, ct.c_double, ct.c_double]
_Clinterp.restype = ct.c_double

_CEqFootprint = libkt17.EqFootprint
_CEqFootprint.argtypes = [np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_CEqFootprint.restype = None

_Cmin = libkt17.min
_Cmin.argtypes = [ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int]
_Cmin.restype = ct.c_double

_CPlanetFootprints = libkt17.PlanetFootprints
_CPlanetFootprints.argtypes = [np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_CPlanetFootprints.restype = None

_CFieldLineLength = libkt17.FieldLineLength
_CFieldLineLength.argtypes = [np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_CFieldLineLength.restype = None

_CRvecs = libkt17.Rvecs
_CRvecs.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_double, ct.c_double, ct.c_double, ct.c_double]
_CRvecs.restype = None

_Ckt17StepRKM = libkt17.kt17StepRKM
_Ckt17StepRKM.argtypes = [ct.c_double, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_double, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_double, ct.c_double, ct.c_double]
_Ckt17StepRKM.restype = None

_Ckt17Trace = libkt17.kt17Trace
_Ckt17Trace.argtypes = [ct.c_double, ct.c_double, ct.c_double, ct.c_int, ct.c_double, ct.c_double, ct.c_int, ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_int,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17Trace.restype = None

_Ckt17MultiTrace = libkt17.kt17MultiTrace
_Ckt17MultiTrace.argtypes = [np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"),np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"),np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, ct.c_int, ct.c_double, ct.c_double, ct.c_int, ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_int,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
_Ckt17MultiTrace.restype = None


# _Ckt17Trace = libkt17.kt17Trace
# _Ckt17Trace.argtypes = [ct.c_double, ct.c_double, ct.c_double, ct.c_int, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_int,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, ct.c_double, ct.c_double, ct.c_double]
# _Ckt17Trace.restype = None

# _Ckt17TraceScaled = libkt17.kt17TraceScaled
# _Ckt17TraceScaled.argtypes = [ct.c_double, ct.c_double, ct.c_double, ct.c_int, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_int,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, ct.c_double, ct.c_double]
# _Ckt17TraceScaled.restype = None

# _Ckt17MultiTrace = libkt17.kt17MultiTrace
# _Ckt17MultiTrace.argtypes = [np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, ct.c_int, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_int,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
# _Ckt17MultiTrace.restype = None

# _Ckt17MultiTraceScaled = libkt17.kt17MultiTraceScaled
# _Ckt17MultiTraceScaled.argtypes = [np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, ct.c_int, ct.c_double, ct.c_double, np.ctypeslib.ndpointer(ct.c_int,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), ct.c_int, np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS"), np.ctypeslib.ndpointer(ct.c_double,flags="C_CONTIGUOUS")]
# _Ckt17MultiTraceScaled.restype = None

